#!/bin/ksh
typeset _target_version=${_target_version:="none"}
typeset _target_release=${_target_release:="none"}
typeset _project_dir=${_project_dir:="none"}
typeset _verbose_mode=${_verbose_mode:=0}
typeset _iso_name=appassure-livedvd-${_target_version}.${_target_release}.iso;               export _iso_name
typeset _disk_label="AppAssureLive";                                                         export _disk_label
typeset _release_notes_url="http://appassure.com";                                           export _release_notes_url
typeset _comm=${_command:="$0"}
typeset _module_livedvd_dir=${_module_livedvd_dir:=$(pwd)/${_comm%/*}}
typeset _module_packaging_dir=${_module_packaging_dir:="none"}

#
# Common variables
#
typeset _packaging_dir="packaging/linux";   export _packaging_dir
typeset _common_dir="common/common.sh";     export _common_dir
typeset _teamcity_home_dir="/home/teamcity"

function help {
   echo "Usage: $0 [-vh] -p <project_dir> -i <mono installation> -n <N> -e <N>"
   echo "  -h - this help screen"
   echo "  -v - verbose all operations"
   echo "  -p - project sources absolute directory"
   echo "  -n - build version"
   echo "  -e - build release" 
}

#
# This function parsing commnad line
#
function parse_command_line {
    while getopts "hvp:i:n:e:" opt; do
        case $opt in
            h)
                help; exit 0
                ;;
            p)
                _project_dir=$OPTARG
                ;;
            n)
                _target_version=$OPTARG
                ;;
            e)
                _terget_release=$OPTARG
                ;;
            v)
	    		_verbose_mode=$OPTARG
		    	;;
            \?)
                help; exit 1
                ;;
        esac
    done
    [ ${_project_dir} == "none" ] && {
        echo "Project dir was not set"
        help
        exit 1
    }
}

#
# We have to parse the command line options before we're including common script
#
parse_command_line $@

#
# Including common functions
#
if [ -e "${_project_dir}/${_packaging_dir}/${_common_dir}" ]; then
    . ${_project_dir}/${_packaging_dir}/${_common_dir}
else
    . ${_command%/*}/../../${_common_dir}
fi

#
# Module variables will be set after parsing command line
#
typeset _module_packaging_dir=${_module_packaging_dir:=$(get_full_path ${_project_dir}/packaging/linux/install)}
typeset _module_livedvd_dir=${_module_livedvd_dir:=$(get_full_path ${_project_dir}/packaging/linux/build/livecd/)}

typeset _os_version=`lsb_release -sc`;  export _os_version
typeset _os_type=`lsb_release -si`;     export _os_type
typeset _normal_user="`id -u`:`id -g`"; export _normal_user

#
# Clearing liveDVD directories from old liveDVD images
#
function remove_old_images
{
    pushd ${_module_livedvd_dir}

    rm -rf ./*.iso
    
    popd
}

#
# Removing temporary directories which were created 
# during livedvd building process
#
function clean {
    logger _debug "Cleaning"
    pushd ${_module_livedvd_dir}

    rm ./chroot.tar.gz
     
    [ -d "chroot" ] && {
        rm -rf chroot
    }

    [ -d "initrd" ] && {
        rm -rf initrd
    }

    [ -d "image" ] && {
        rm -rf image
    }
    
    popd #${_module_livedvd_dir}/
}

function success_or_die {
    [ $? -eq 0 ] || {
         logger _error "$1"
         exit 1
    } 
}

function get_file_structure {
    
    logger _debug "Getting file structure from tar"
    
    # We're expecting chroot.tar.gz in teamcity home directory.
    # In case if it's not there - look in current dir 
    if [ -e $_teamcity_home_dir/chroot.tar.gz ] ; then
        chroot_source=$_teamcity_home_dir/chroot.tar.gz
    elif [ -e ~/chroot.tar.gz ]; then
        chroot_source=~/chroot.tar.gz
    else
        logger _error "Cannot get a chroot.tar.gz"
        exit 1
    fi
    cp ${chroot_source} ${_module_livedvd_dir}/
     
    if [ ! -e ${_module_livedvd_dir}/chroot.tar.gz ]; then
        logger _error "Can not find chroot.tar.gz"
        exit 1
    fi

    # unpacking chroot.tar.gz
    pushd ${_module_livedvd_dir}
    tar xpzf chroot.tar.gz 
    
    popd #${_module_livedvd_dir}

}

#
# We have to patch Agent config during after agent install 
# to switch it to Rrc mode. 
#
function patch_app_config {
    app_config_path=${_module_livedvd_dir}/chroot/opt/appassure/aagent/Agent.Service.Mono.exe.config 
    
    PREVVALUE="^.*IsAgentRecoveryConsoleInstance.*$"
    NEXTVALUE="    <add key=\"IsAgentRecoveryConsoleInstance\" value=\"true\" \/>"
 				
    sed "s/$PREVVALUE/$NEXTVALUE/g" <$app_config_path >$app_config_path.new
    mv $app_config_path.new $app_config_path
}

function install_agent_from_installer {
    logger _info "Installing aagent using installer" 
    resources_dir=${_module_livedvd_dir}/resources
    chroot_dir=${_module_livedvd_dir}/chroot
    appassure_installer_base_name="appassure-installer_ubuntu*"
    chroot_script_name="RunInstall.sh"

    cp /etc/hosts /etc/resolv.conf  $chroot_dir/etc/   &&\
    cp /etc/apt/sources.list     $chroot_dir/etc/apt/
    success_or_die "Can't copy /etc/hosts | /etc/resolv.conf | /etc/apt/sources.list"

    # Copy custom init. This scrip will be executed on start and perform final 
    # tuning of the liveDVD
    cp $resources_dir/custom_init $chroot_dir/etc/init.d
 
    # This is a small scpirt to show appassure hello page on start. 
    cp $resources_dir/run_bash $chroot_dir/usr/bin
 
    # Copy verison file to proc. Needed to prevent mounting /proc during chrooting.
    cp $resources_dir/version $chroot_dir/proc
  
    # Copy the AppAssure installer; 
    agent_installer=`ls $_module_packaging_dir/$appassure_installer_base_name 2>/dev/null | tail -1`
    agent_installer_file_name=$(basename $agent_installer)

    [ -n "$agent_installer" ] || {
       logger _error  "No AppAssure installer found"
       exit 1
    }

    cp $agent_installer $chroot_dir/
    
    # Perform install of agent in chroot 
    cat > $chroot_dir/$chroot_script_name  << SCRIPT
#!/bin/bash
chmod +x /$agent_installer_file_name
chmod +x /etc/init.d/custom_init
chmod +x /usr/bin/run_bash
/$agent_installer_file_name -f

exit 0
SCRIPT
    
    chmod +x $chroot_dir/$chroot_script_name

    chroot $chroot_dir /$chroot_script_name
    success_or_die "chroot failed"


    #Remove installer script
    rm $chroot_dir/$chroot_script_name
    #Remove installer from chroot
    rm -r $chroot_dir/$agent_installer_file_name
    #Remove proc/version from chroot
    rm $chroot_dir/proc/version
}

#
# Theese function wait for prepared chroot directory
# and pack it to the resulting *.iso image. 
#
function pack_chroot_to_iso {
    logger _debug "Creating livecd from file structure"
    RESOURCES=${_module_livedvd_dir}/resources
    CHROOT=${_module_livedvd_dir}/chroot
    #
    #  After Casper installation: customize the default user
    #
    [ -r "$RESOURCES/casper.conf" ] \
        && cp $RESOURCES/casper.conf    $CHROOT/etc/      \
        || logger _error "casper.conf not found in $RESOURCES"
    
    pushd $_module_livedvd_dir
    logger _debug "Populating the image/ directory"
    mkdir -p image/{casper,isolinux,install}
    
    cp $CHROOT/vmlinuz    image/casper/vmlinuz  && \
    cp $CHROOT/initrd.img image/casper/initrd.gz
    success_or_die "initrd image not found in the chroot"
    
    [ -d initrd ] && rm -r initrd
    logger _debug "Patching the livecd's initrd"
    mkdir initrd

    pushd initrd

    (gunzip -dc ../image/casper/initrd.gz | cpio -imd --no-absolute-filenames)

    popd #initrd

     [ -r "$RESOURCES/casper.conf" ] \
        && cp $RESOURCES/casper.conf    initrd/etc/      \
        || logger _error "casper.conf not found in $RESOURCES"

    pushd initrd

    (find . | cpio -o -H newc | gzip -9 > initrd.gz )

    popd #initrd
    mv initrd/initrd.gz image/casper/

    cp $RESOURCES/isolinux.bin               image/isolinux/     &&\
    cp $RESOURCES/memtest86+.bin             image/install/memtest
    success_or_die "isolinux.bin or memtest not found on the host"

    cp $RESOURCES/isolinux.txt          image/isolinux/     &&\
    cp $RESOURCES/isolinux.cfg          image/isolinux/
    success_or_die "Bootloader resources not found, set \$RESOURCES"

    chroot $CHROOT dpkg-query -W --showformat='${Package} ${Version}\n' |  tee image/casper/filesystem.manifest
    success_or_die "chroot  dpkg-query failed"

    logger _debug "Compressing the filesystem image..."
    rm -f image/casper/filesystem.squashfs
    mksquashfs $CHROOT    image/casper/filesystem.squashfs -e boot
    success_or_die "mksquashfs failed"

    printf `du -sx --block-size=1 $CHROOT | cut -f1` > image/casper/filesystem.size

    cat > image/README.diskdefines << EOF
#define DISKNAME        $DISK_LABEL
#define TYPE            binary
#define TYPEbinary      1
#define ARCH            $ISO_ARCH
#define ARCH$ISO_ARCH   1
#define DISKNUM         1
#define DISKNUM1        1
#define TOTALNUM        0
#define TOTALNUM0       1
EOF

    echo "---- Writing the LiveCD metainformation"
    touch image/ubuntu
    mkdir image/.disk/
    
    pushd image/.disk
    
    touch base_installable
    echo "full_cd/single" > cd_type
    echo "$DISK_LABEL"  > info
    echo "$RELEASE_NOTES_URL" > release_notes_url
    
    popd #image/.disk

    echo "---- Calculating MD5 hashes"
    (cd image && find . -type f -print0 | xargs -0  md5sum | grep -v "\./md5sum.txt" > md5sum.txt)

    echo "---- Creating the image..."
    
    pushd image
    
    genisoimage -r -V "$DISK_LABEL" -cache-inodes -J -l -b isolinux/isolinux.bin \
            -c isolinux/boot.cat -no-emul-boot -boot-load-size 4 -boot-info-table -o \
            ../$_iso_name .
    success_or_die "genisoimage failed"
    
    popd #image

    popd #$_module_livedvd_dir
}

function md5_it {
    pushd $_module_livedvd_dir
    md5sum $_iso_name >> $_project_dir/packaging/linux/install/artifacts.md5
    popd
}

function build_livecd {
    logger _debug "Performing liveCD build"

    get_file_structure

    install_agent_from_installer

    patch_app_config    

    pack_chroot_to_iso

    md5_it
}

#
#   Main entry point
#

logger _info "Welcome to AppAssureLiveCD build system"

# check the host distribution
[ ${_os_type} == Ubuntu ] || {
    echo "Building livecd is supported on Ubuntu only"
    exit 1
}

if [ $(whoami) != "root" ]; then
    echo "Started not under root user"
    exec sudo -E $0 $@  
else
    echo "Starting build Appassure LiveDVD"
    
    #Removing old *.iso files    
    remove_old_images

    build_livecd

    clean
fi
